/*
 * Copyright (C) 2015 Darron Broad
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#define WIDTH (12)

int
main(int argc, char **argv)
{
	struct stat st;
	uint8_t *mem;
	int fd, i, size, addr = 0;

	if (argc != 4) {
		printf("Try: %s var-name bin-file bin-addr\n", argv[0]);
		exit(1);
	}
	if (stat(argv[2], &st) != 0) {
		printf("stat failed");
		exit(1);
	}
	size = (int)st.st_size;
	if ((mem = (uint8_t *)calloc(size, sizeof(uint8_t))) == NULL) {
		printf("calloc failed");
		exit(1);
	}
	if ((fd = open(argv[2], O_RDONLY)) < 0) {
		printf("open failed");
		exit(1);
	}
	if ((read(fd, (void *)mem, size)) != size) {
		printf("read failed");
		exit(1);
	}
	close(fd);

	addr = strtoul(argv[3], NULL, 0);
	printf("#define %s_ADDR (0x%04X)\n", argv[1], addr);

	printf("#define %s_SIZE (%d)\n", argv[1], size);
	printf("const uint8_t %s[%s_SIZE] = { /* %s */\n", argv[1], argv[1], argv[2]);
	for(i = 0; i < size;) {
		printf("0x%02X,", mem[i]);
		if ((++i % WIDTH) == 0)
			putchar('\n');
	}
	if (i % WIDTH)
		putchar('\n');
	printf("};\n");

	free(mem);
	exit(0);
}
